function images = pco_adaptor_example_callback(imcount, average_count, ...
    dispose_count, exp_start, exp_step, timeout_s)
% This function records and returns a series of mean images for different
% exposure times. Therefore a callback function is triggerd periodically
% which increases the exposure time by a certain step. After acquisition
% finished, the mean images for all exposure times are computed and shown.
%
%   [images] = pco_adaptor_example_callback(imcount, average_count, ...
%       dispose_imcount, exp_start, exp_step, timeout_s)
%
% * Input parameters
%       imcount:                    Number of mean images to acquire
%       average_count:              Number of frames to be averaged
%                                   frames acquired event is generated                              
%       dispose_count:              Number of throwaway images for every
%                                   average image calculation
%       exp_start:                  Initial exposure time in [s] that is 
%                                   set at the beginning of the function
%       exp_step:                   Exposure time step in [s] that is used 
%                                   to increase the exposuer time at every
%                                   callback function call
%       timeout_s:                  Number of seconds to wait before video
%                                   input timeout occurs
%
% * Output parameters
%       images:                     Stack with calculated mean images
%
% - Set property values and trigger mode
% - Start preview, configure callback, stop preview
% - Trigger callback function and increase exposure time while acquiring 
%   required images 
% - Calculate for every exposure time the mean image 
% - Display mean images and reset the adaptor
%

%% Set default values for paramters that haven't been set
if(~exist('imcount','var'))
    imcount = 10;
end

if(~exist('average_count','var'))
    average_count = 50;
end

if(~exist('dispose_count','var'))
    dispose_count = 5;
end

if(~exist('exp_start','var'))
    exp_start = 0.001;
end

if(~exist('exp_step','var'))
    exp_step = 0.001;
end

if(~exist('timeout_s','var'))
    timeout_s = 120;
end

%% Get adaptor name
%Identify required adaptor
if verLessThan('matlab','9.0') %older than R2016a
  error('This adaptor is supported in Matlab R2016a and later versions'); 
else %R2016a and newer
  if(strcmp(computer('arch'),'win64'))
    adaptorName = ['pcocameraadaptor_r' version('-release')];
  elseif(strcmp(computer('arch'),'glnxa64'))
    adaptorName = ['pcocameraadaptor_r' version('-release')];
  else
    error('This platform is not supported.');
  end
end

%% Configure camera and record images
%Create video input object
vid = videoinput(adaptorName, 0);

%Create adaptor source
src = getselectedsource(vid);

%Set timestamp mode
src.TMTimestampMode = 'BinaryAndAscii';

%Set initial exposure time
src.ExposureTime_s = exp_start;

%Set logging to memory
vid.LoggingMode = 'memory';

%Set the timeout property of video input object 'vid'
set(vid,'Timeout',timeout_s); 

%Set frames per trigger
vid.FramesPerTrigger = average_count + dispose_count;

%Set trigger mode
triggerconfig(vid, 'immediate');

% TriggerRepeat is zero based and is always 
% one less than the number of triggers.
vid.TriggerRepeat = imcount - 1;

%Set frame number at which the frames acquired event is generated
vid.FramesAcquiredFcnCount = vid.FramesPerTrigger;

%Connect the frames acquired event to the callback function
vid.FramesAcquiredFcn = {@callbackFunc,exp_step};

%Start acquisition
start(vid);

%Start preview
preview(vid);

%Read out all images and remove them from memory buffer
%Function blocks until the images have been acquired
images_temp = getdata(vid, vid.FramesPerTrigger * (vid.TriggerRepeat + 1));

%Stop preview
stoppreview(vid);

%% Compute mean images for every exposure step and show images
for i=0:(imcount - 1)
    
    %Write all images with the same exposure time to a variable
    images_same_exp = images_temp(:,:,:, ...
        (i*vid.FramesPerTrigger + dispose_count + 1) ...
        :(i+1) * vid.FramesPerTrigger);

    %Calculate mean image
    meanImage = uint16(mean(images_same_exp,4));
    
    if i==0
        %For first image assign it to image vector
        images = meanImage;
    else
        %Append new meanImage to vector
        images = cat(4,images,meanImage);
    end
end   
   
%Show mean images in Video Viewer app
implay(images);

%Reset adaptor
imaqreset;

end

%% Callback function, triggered by the 'vid.FramesAcquiredFcn' event
function callbackFunc(obj,event,exp_step)

	%Get adaptor source
	src = getselectedsource(obj);

	%Increase exposure time
	src.ExposureTime_s = src.ExposureTime_s + exp_step;

	%Get frame number from event log
	frameNr = event.Data.FrameNumber;

	%Show frame number
	sprintf('%s %d','FrameNumber', frameNr)

end