function flim_example_file(refFileName, sampleFileName, f, tauRef, ...
    nPhases, nBits, bitAlignDiv, phaseNumber, addPhaseSampling, ...
    phaseOrder, tapSelection, asymCorrection, dispMinLifetime, ...
    dispMaxLifetime)

%
% function flim_example_file(refFileName, sampleFileName, f, tauRef, ...
%     nPhases, nBits, bitAlignDiv, phaseNumber, addPhaseSampling, ...
%     phaseOrder, tapSelection, asymCorrection, dispMinLifetime, ...
%     dispMaxLifetime)
%
% This function demonstrates the use of FLIM functions to obtain lifetime
% images out of image stacks stored in multi-TIFF files.
%
% Input parameters:
%
% refFileName:      Filename of reference image stack
% sampleFileName:   Filename of sample image stack
% f:                Modulation frequency (in Hertz)
% tauRef:           Lifetime of reference (in seconds)
% nPhases:          Number of equidistant phases per modulation period
% nBits:            Bit resolution of image stack data
% bitAlignDiv:      Denominator for bit alignment correction
% phaseNumber:      Selects the number of equidistant phase samples per
%                   modulation period of 360 degrees (2, 4, 8 or 16).
%                   Additionally, it offers a 'shiftable pair' mode in
%                   which taps A and B can be manually phase-shifted
%                   relative to the modulation signal.
%                   Type string: 'shiftablePair', '02', '04', '08', '16'
% addPhaseSampling: If set to 'yes' all phase samples are sampled by both
%                   taps A and B, doubling the total number of phase
%                   images. It is used for asymmetry correction of the
%                   taps.
%                   Type string: 'no', 'yes'
% phaseOrder:       Determines the arrangement of adjacent phase pairs. If
%                   'opposite' is selected all complementary phase pairs
%                   are sorted adjacently. If 'ascending' is selected the
%                   phase pairs cover the modulation period linearly. This
%                   selection is meaningful if 'Additional Phase Sampling'
%                   is set to 'yes'.
%                   Type string: 'ascending', 'opposite'
% tapSelection:     Determines which taps are read (Tap A & B, Tap A or
%                   Tap B).
%                   Type string: 'Tap A', 'Tap B', 'Tap A+B'
% asymCorrection:   If switched 'on' the asymmetry correction is performed
%                   within the camera by averaging corresponding phase
%                   images of taps A and B.
%                   This mode requires the following settings:
%                   - phaseNumber:      all except 'shiftablePair'
%                   - addPhaseSampling: 'yes',
%                   - phaseOrder:       'opposite'
%                   - tapSelection:     'Tap A+B'
%                   Type string: 'on', 'off'
% dispMinLifetime:  Minimum lifetime for display (in seconds)
% dispMaxLifetime:  Maximum lifetime for display (in seconds)
%

    % #####################################################################
    % 1) read image stacks from files (reference and sample)
    % #####################################################################

    refStack = flim_rearrange_stack(read_img_stack_from_file(...
        refFileName, bitAlignDiv), phaseNumber, addPhaseSampling, ...
        phaseOrder, tapSelection, asymCorrection);

    sampleStack = flim_rearrange_stack(read_img_stack_from_file(...
        sampleFileName, bitAlignDiv), phaseNumber, addPhaseSampling, ...
        phaseOrder, tapSelection, asymCorrection);

    % #####################################################################
    % 2) compute phasor images (reference and sample)
    % #####################################################################

    [phasorRef normIntRef] = ...
        flim_numeric_harmonic_analysis_phasor(refStack, nBits, ...
            nPhases, 1);

    [phasorSample normIntSample] = ...
        flim_numeric_harmonic_analysis_phasor(sampleStack, nBits, ...
            nPhases, 1);

    % #####################################################################
    % 3) compute referenced phasor image
    % #####################################################################

    phasor = flim_referencing_phasor(phasorSample, phasorRef, tauRef, f);

    % #####################################################################
    % 4) compute lifetime images based on modulation index and phase
    % #####################################################################

    [mLifetime pLifetime] = flim_get_lifetimes_phasor(phasor, f);

    % #####################################################################
    % 5) display results
    % #####################################################################

    % normalized intensity image of sample
    figure('Name', 'Sample - Normalized Intensity', 'NumberTitle', 'off');

    clims = [min(min(normIntSample)) max(max(normIntSample))];
    colormap('gray');
    imagesc(normIntSample, clims);
    axis off;
    axis image;

    % modulation index lifetime image
    figure('Name', 'Sample - Modulation Index Lifetime', 'NumberTitle', ...
           'off');

    clims = [dispMinLifetime dispMaxLifetime];
    mLifetimeRGB = ind2rgb(gray2ind(mat2gray(real(mLifetime), clims), ...
        256), jet(256));

    image(mLifetimeRGB .* repmat(normIntSample, 1, 1, 3));
    axis off;
    axis image;
    colormap(jet(256));
    caxis(clims);
    colorbar;
    ylabel(colorbar, '\tau_m [s]');

    % phase lifetime image
    figure('Name', 'Sample - Phase Lifetime', 'NumberTitle', 'off');

    clims = [dispMinLifetime dispMaxLifetime];
    pLifetimeRGB = ind2rgb(gray2ind(mat2gray(pLifetime, clims), ...
        256), jet(256));

    image(pLifetimeRGB .* repmat(normIntSample, 1, 1, 3));
    axis off;
    axis image;
    colormap(jet(256));
    caxis(clims);
    colorbar;
    ylabel(colorbar, '\tau_{\phi} [s]');

end
